#!/bin/sh

# cu audio upgrade script

usage()
{
cat << EOF
    usage: $0 options

    This script writes cu audio images to hardware

    OPTIONS:
       -h      show this help text
       -r      update rootfs <filename>
       -p      path to progress file
       -s      path to status file
EOF
}

XPRE="cu-audio_upgrade >>"

ROOTFS_FILE="rootfs.tar.xz"
PROGRESS_FILE=$(tty)    # path to progress file
STATUS_FILE="/dev/null" # path to status file

STEPS_TOTAL=4
UPGRADE_STEP=0

print_error()
{
    printf "%s ERROR    step %s/%s - %s\n" "$XPRE" "$UPGRADE_STEP" "$STEPS_TOTAL" "$1" >> $PROGRESS_FILE
	printf "{\"done\":true,\"error\":\"true\"}" > $STATUS_FILE
    exit 1
}

print_progress()
{
    printf "%s PROGRESS step %s/%s - %s\n" "$XPRE" "$UPGRADE_STEP" "$STEPS_TOTAL" "$1" >> $PROGRESS_FILE
	printf "{\"done\":false,\"progress\":\"%s\"}" "$1" > $STATUS_FILE
}

O=`getopt -q -- hr:p:s: "$@"`
eval set -- "$O"
while true; do
	case $1 in
		-h)
			usage
			exit 0
			shift;;
		-r)
			ROOTFS_FILE=$2
			shift 2;;
		-p)
			PROGRESS_FILE=$2
			shift 2;;
		-s)
			STATUS_FILE=$2
			shift 2;;
		--)
			shift; break;;
		*)
			shift; break;;
	esac
done

print_progress "File: ${ROOTFS_FILE}"

BOARD_TYPE=$(cat /etc/board)

if [ "$BOARD_TYPE" != "cu_audio" ] && \
   [ "$BOARD_TYPE" != "cu_audio_golden" ]
then
	print_error "This upgrade can only be installed on a CU audio."
	exit 1
fi

# Note: an update will change the application - number
# The number is stored to a temporary file (removed on reboot)
# to make sure we don't accidently update the wrong location
# in case a second update is done, without rebooting.
if [ ! -e /tmp/application ]
then
	fw_printenv -n application > /tmp/application
fi
APPLICATION=$(cat /tmp/application)

# currently first partition?
if [ $APPLICATION -ne 1 ]; then
	ROOTFS_PARTITION="/dev/mmcblk1p1"
	APPLICATION=1
	ROOTFSLABEL="rootfsA"
else
	ROOTFS_PARTITION="/dev/mmcblk1p2"
	APPLICATION=2
	ROOTFSLABEL="rootfsB"
fi;

if [ -z "$ROOTFS_PARTITION" ]
then
	print_error "Board is missing the required partition."
	exit 1
fi

UPGRADE_STEP="1"
print_progress "Formatting: ${ROOTFS_PARTITION}"
mkfs.ext4 -E nodiscard -F -L ${ROOTFSLABEL} ${ROOTFS_PARTITION} &> /dev/null

UPGRADE_STEP="2"
print_progress "Installing new rootfs"
rm -rf /tmp/rootfs
mkdir -p /tmp/rootfs
mount -o nodiscard,noatime,nodiratime,data=journal ${ROOTFS_PARTITION} /tmp/rootfs

tar -xf ${ROOTFS_FILE} -C /tmp/rootfs
sync
umount /tmp/rootfs

UPGRADE_STEP="3"
print_progress "Update uboot variables: application $APPLICATION"
fw_setenv application $APPLICATION
fw_setenv bootdelay 0
sync

# less clean fix -> explicitly move file if it exists
mv /home/root/cu_config.txt /home/root/config/cu_config.txt
sync

UPGRADE_STEP="4"
print_progress "Update finished"

exit 0
